-- ****************************************************************************************************
-- Change Log:
-- Version 5.0.240116
--
-- 2024-01-16
-- Increased version number for new major build (I should have done this on the last build...)
-- Fixed a bug that caused a LUA error on the first time the character pane is opened per toon login
--
-- 2024-01-03
-- Updated interface number for 10.2.5
-- Added ShowWeapons feature with toggle in Interface Options to show weapon(s) iLvl in the tooltip
--
-- 2023-07-29
-- Updated interface number for 10.1.5
-- Removed spec display since that's part of Blizzard's default UI now
--
-- 2023-01-28
-- Updated interface number for 10.0.5
-- Reworked logic on the tooltip's OnUpdate hook to prevent stack overflow errors.
--
-- 2022-12-25
-- Added logic to keep ilvl and spec info on the tooltip when it redraws if the target hasn't changed.
-- Added a bunch of round-about logic to make the redraw logic work properly.
-- Honestly, I lost track of all the changes that I had to make here in order to make this work - this
--   was NOT an easy problem do solve. Many thanks to the Astral Keys developers for their code
--   pointing me in the right direction to make this work.
-- Also added features to optionally disable use of class colors or showing the target's spec
--
-- 2022-11-17
-- Updated how the "Add Line" checkbox saves it's value to the variable.
--
-- 2022-11-15
-- Updated interface number for 10.0.2
-- Added support for Evoker
--
-- 2022-10-26
-- Updated interface number for 10.0.0
--
-- 2022-06-07
-- Updated interface number for 9.2.5 (sorry for never updating to 9.2.0 - life got the better of me
-- for a while there...)
--
-- 2021-11-02
-- Update interface number for 9.1.5
--
-- 2021-07-02
-- Updated version and interface numbers for 9.1
--
-- 2021-03-11
-- Updated version and interface numbers for 9.0.5
--
-- 2020-12-30
-- Updated the variables load logic to be based on the VARIABLES_LOADED event
-- Corrected tooltip on Interface > Addons page to show better explanation of the feature
--
-- 2020-12-29
-- Added Interface > Addons page for Khes'yc iLvl with toggle to add a blank line before
-- details added by Khes'yc iLvl
--
-- 2020-11-26
-- Updated KhesyciLvlGetClassColor to use classId instead of className to work in different
-- language versions of the game
--
-- 2020-11-18
-- Updated interface build number for 9.0.2
--
-- 2020-09-27
-- Initial build. Scans other players for spec and iLvl.
-- Limited distribution for personal use only.
-- ****************************************************************************************************

--slash commands
local ki_debug = false;
SLASH_KIDEBUG1 = "/kidebug";
SlashCmdList["KIDEBUG"] = function(msg)
	if (ki_debug) then
		print("Khes'yc iLvl - Debug Mode: Disabled");
		ki_debug = false;
	else
		print("Khes'yc iLvl - Debug Mode: Enabled");
		ki_debug = true;
	end
end

--activate dev mode - caution: not all features will work fully
local ki_dev = false;

--declarations
local addon = ...;
local addonTitle = GetAddOnMetadata(addon, "Title");
local addonVersion = GetAddOnMetadata(addon, "Version");
local isWaiting = false;
local name = "";
local scanName = "";
local realm = "";
local scanRealm = "";
local iLvl = "0";
local weapiLvl = "0";
local gear = {};
local playerGear = {};
local slots = {"MainHand", "SecondaryHand", "Head", "Neck", "Shoulder", "Back", "Chest", "Wrist", "Hands", "Waist", "Legs", "Feet", "Finger0", "Finger1", "Trinket0", "Trinket1"};
local enchants = {};
local gems = {};

--GLOBALS
local _G = _G;
local InterfaceOptions_AddCategory = _G.InterfaceOptions_AddCategory;

--Create the Blizzard addon option frame
local panel = CreateFrame("Frame", addonTitle .. "BlizzOptions");
panel.name = addonTitle;
InterfaceOptions_AddCategory(panel);

local fs = panel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
fs:SetPoint("TOPLEFT", 10, -15);
fs:SetJustifyH("LEFT");
fs:SetJustifyV("TOP");
fs:SetText(addonTitle);

local st = panel:CreateFontString(nil, "OVERLAY", "SystemFont_Outline");
st:SetPoint("TOPLEFT", 10, -48);
st:SetPoint("BOTTOMRIGHT", panel, "TOPRIGHT", 10, -80);
st:SetJustifyH("LEFT");
st:SetJustifyV("TOP");
st:SetText("Please consider supporting this and our other projects at www.khesyc.com via Patreon at www.patreon.com/khesyc or Venmo at @Khesyc.");

local chkShowTooltip = CreateFrame("CheckButton", "ki_chkShowTooltip", panel, "ChatConfigCheckButtonTemplate");
chkShowTooltip:SetPoint("TOPLEFT", 10, -85);
chkShowTooltip.tooltip = "Add item levels to character tooltips?";
chkShowTooltip:SetScript("OnClick", function(self)
	ki_showTooltip = chkShowTooltip:GetChecked();
end)
local lblShowTooltip = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblShowTooltip:SetPoint("TOPLEFT", 35, -90);
lblShowTooltip:SetJustifyH("LEFT");
lblShowTooltip:SetJustifyV("TOP");
lblShowTooltip:SetText("Add iLvl to tooltips?");

local chkAddLine = CreateFrame("CheckButton", "ki_chkAddLine", panel, "ChatConfigCheckButtonTemplate");
chkAddLine:SetPoint("TOPLEFT", 35, -110);
chkAddLine.tooltip = "Add a blank line before Khes'yc iLvl details?";
chkAddLine:SetScript("OnClick", function(self)
	ki_addLine = chkAddLine:GetChecked();
end)
local lblAddLine = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblAddLine:SetPoint("TOPLEFT", 60, -115);
lblAddLine:SetJustifyH("LEFT");
lblAddLine:SetJustifyV("TOP");
lblAddLine:SetText("Add Blank Line?");

local chkShowWeapons = CreateFrame("CheckButton", "ki_chkShowWeapons", panel, "ChatConfigCheckButtonTemplate");
chkShowWeapons:SetPoint("TOPLEFT", 35, -135);
chkShowWeapons.tooltip = "Show weapon(s) after iLvl?";
chkShowWeapons:SetScript("OnClick", function(self)
	ki_showWeapons = chkShowWeapons:GetChecked();
end)
local lblShowWeapons = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblShowWeapons:SetPoint("TOPLEFT", 60, -140);
lblShowWeapons:SetJustifyH("LEFT");
lblShowWeapons:SetJustifyV("TOP");
lblShowWeapons:SetText("Show Weapons?");

local chkShowCharacter = CreateFrame("CheckButton", "ki_chkShowCharacter", panel, "ChatConfigCheckButtonTemplate");
chkShowCharacter:SetPoint("TOPLEFT", 10, -165);
chkShowCharacter.tooltip = "Add Khes'yc iLvl to character screen?";
chkShowCharacter:SetScript("OnClick", function(self)
	ki_showCharacter = chkShowCharacter:GetChecked();
end)
local lblShowCharacter = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblShowCharacter:SetPoint("TOPLEFT", 35, -170);
lblShowCharacter:SetJustifyH("LEFT");
lblShowCharacter:SetJustifyV("TOP");
lblShowCharacter:SetText("Add item data to character screen?");

local chkInclLevel = CreateFrame("CheckButton", "ki_chkShowLevel", panel, "ChatConfigCheckButtonTemplate");
chkInclLevel:SetPoint("TOPLEFT", 35, -190);
chkInclLevel.tooltip = "Include item level?";
chkInclLevel:SetScript("OnClick", function(self)
	ki_showLevel = chkInclLevel:GetChecked();
end)
local lblInclLevel = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblInclLevel:SetPoint("TOPLEFT", 60, -195);
lblInclLevel:SetJustifyH("LEFT");
lblInclLevel:SetJustifyV("TOP");
lblInclLevel:SetText("Include Item Level?");

local chkInclDura = CreateFrame("CheckButton", "ki_chkShowDura", panel, "ChatConfigCheckButtonTemplate");
chkInclDura:SetPoint("TOPLEFT", 35, -215);
chkInclDura.tooltip = "Include item durability?";
chkInclDura:SetScript("OnClick", function(self)
	ki_showDura = chkInclDura:GetChecked();
end)
local lblInclDura = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblInclDura:SetPoint("TOPLEFT", 60, -220);
lblInclDura:SetJustifyH("LEFT");
lblInclDura:SetJustifyV("TOP");
lblInclDura:SetText("Include Item Durability?");

local chkInclGems = CreateFrame("CheckButton", "ki_chkShowGems", panel, "ChatConfigCheckButtonTemplate");
chkInclGems:SetPoint("TOPLEFT", 35, -240);
chkInclGems.tooltip = "Include missing gems and enchants?";
chkInclGems:SetScript("OnClick", function(self)
	ki_showGems = chkInclGems:GetChecked();
end)
local lblInclGems = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
lblInclGems:SetPoint("TOPLEFT", 60, -245);
lblInclGems:SetJustifyH("LEFT");
lblInclGems:SetJustifyV("TOP");
lblInclGems:SetText("Include Missing Gems/Enchants?");

--Create event handler frame
local ki_Event = CreateFrame("frame", "ki_Event");
ki_Event:RegisterEvent("VARIABLES_LOADED");
ki_Event:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");

ki_Event:SetScript("OnEvent", function(self, event, ...)
	if (event == "VARIABLES_LOADED") then
		KhesyciLvlInit();
		ki_Event:UnregisterEvent("VARIABLES_LOADED");
		ki_Event:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	elseif (event == "UPDATE_MOUSEOVER_UNIT") then
		if not isWaiting then
			KhesyciLvlCheckName();
		end
	elseif (event == "INSPECT_READY") then
		KhesyciLvlInspect(...);
		ki_Event:UnregisterEvent("INSPECT_READY");
		ki_Event:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	elseif (event == "PLAYER_EQUIPMENT_CHANGED") then
		KhesyciLvlScanPlayer();
	end
end);

--Initialize
function KhesyciLvlInit()
	if (ki_showTooltip == nil) then ki_showTooltip = true end
	if (ki_addLine == nil) then ki_addLine = true end
	if (ki_showWeapons == nil) then ki_showWeapons = true end
	if (ki_showCharacter == nil) then ki_showCharacter = true end
	if (ki_showLevel == nil) then ki_showLevel = true end
	if (ki_showDura == nil) then ki_showDura = true end
	if (ki_showGems == nil) then ki_showGems = true end

	chkShowTooltip:SetChecked(ki_showTooltip);
	chkAddLine:SetChecked(ki_addLine);
	chkShowWeapons:SetChecked(ki_showWeapons);
	chkShowCharacter:SetChecked(ki_showCharacter);
	chkInclLevel:SetChecked(ki_showLevel);
	chkInclDura:SetChecked(ki_showDura);
	chkInclGems:SetChecked(ki_showGems);

	--set slots that have enchants/gems --note: this will have to be updated by patch
	enchants[1] = 1; --head
	gems[1] = 1;
	enchants[15] = 1; --cloak
	enchants[5] = 1; --chest
	enchants[9] = 1; --bracers
	gems[9] = 1;
	enchants[6] = 1; --belt
	gems[6] = 1;
	enchants[7] = 1; --legs
	enchants[8] = 1; --boots
	enchants[16] = 1; --weapon
	enchants[17] = 1; --off hand
	gems[2] = 3; --neck
	gems[11] = 1; --ring(s)
	gems[12] = 1;

	--begin loop to keep the mouseover event registered if there's a failure in the event registration logic
	KhesyciLvlRegisterMouseover();
end

function KhesyciLvlRegisterMouseover()
	ki_Event:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	C_Timer.After(5, function() KhesyciLvlRegisterMouseover() end);
end

function KhesyciLvlGetMouseoverName()
	local mouseName, mouseRealm = UnitName("mouseover");
	if (mouseRealm == nil or mouseRealm == '') then
		mouseRealm = GetRealmName();
	end
	return mouseName, mouseRealm;
end

function KhesyciLvlVerifyScanned(scannedName, scannedRealm)
	--if (ki_debug) then print("Khes'yc iLvl VerifyScanned()") end
	if (scannedName == scanName and scannedRealm == scanRealm) then
		--scanned target is the same, validation passes
		return true;
	else
		--target has changed, snapshot the new target and reset any scanned data
		scanName = scannedName;
		scanRealm = scannedRealm;

		name = "";
		realm = "";
		iLvl = "0";
		weapiLvl = "0";
		gear = {};

		return false;
	end
end

function KhesyciLvlSetWaiting()
	isWaiting = true;
	C_Timer.After(0.1, function() KhesyciLvlClearWaiting() end);
end

function KhesyciLvlClearWaiting()
	isWaiting = false;
	KhesyciLvlCheckName();
end

function KhesyciLvlCheckName()
	if (UnitIsPlayer("mouseover") and isWaiting == false) then
		local newName, newRealm = KhesyciLvlGetMouseoverName();
		if not KhesyciLvlVerifyScanned(newName, newRealm) then
			--target is a new player - initiate wait to verify if cursor is just "passing over" target
			if (ki_debug) then print("Khes'yc iLvl: Check Name - New Target") end
			KhesyciLvlSetWaiting();
		else
			--we still have the same target - initiate inspect and get spec and item level information
			if (scanName ~= name or scanRealm ~= realm) then
				if (ki_debug) then print("Khes'yc iLvl: Check Name - New Inspect") end
				--submit for inspect and register for return
				NotifyInspect("mouseover");
				ki_Event:RegisterEvent("INSPECT_READY");
				ki_Event:UnregisterEvent("UPDATE_MOUSEOVER_UNIT");
			else
				if (ki_debug) then print("Khes'yc iLvl: Check Name - Existing Inspect") end
			end
		end
	end
end

function KhesyciLvlInspect(guid)
	if (ki_debug) then print("Khes'yc iLvl Inspect()") end
	if UnitIsPlayer("mouseover") then
		--get the current mouseover target
		local newName, newRealm = KhesyciLvlGetMouseoverName();

		--make sure that our mouseover target still matches our inspect target
		if KhesyciLvlVerifyScanned(newName, newRealm) then
			--snapshot current name, class, spec, iLvl into global variables
			name = newName;
			realm = newRealm;

			_, _, class = UnitClass("mouseover");
			iLvl = C_PaperDollInfo.GetInspectItemLevel("mouseover");

			for i = 1, #slots do
				local itemLevel, iqRed, iqGreen, iqBlue;
				local itemLink = GetInventoryItemLink("mouseover", GetInventorySlotInfo(slots[i] .. "Slot"));
				if (itemLink ~= nil ) then
					local itemType = select(9, GetItemInfo(itemLink));
					local item = Item:CreateFromItemLink(itemLink);
					item:ContinueOnItemLoad(function()
						itemLevel = item:GetCurrentItemLevel();
						iqRed, iqGreen, iqBlue = item:GetItemQualityColorRGB();
					end)
					local gearItem = {slots[i], itemLink, itemLevel, iqRed, iqGreen, iqBlue, itemLink};
					gear[slots[i]] = gearItem;
				end
			end
			
			if (gear["MainHand"] ~= nil) then
				local mainiLvl = gear["MainHand"][3];
				if (gear["SecondaryHand"] ~= nil) then
					mainiLvl = mainiLvl .. "/" .. gear["SecondaryHand"][3];
				end

				weapiLvl =  " (" .. mainiLvl .. ")";
			end
		end
	end
end

function KhesyciLvlScanPlayer()
	if ki_debug then print("Khes'yc iLvl Scan Player()") end
	playerGear = {};
	for i = 1, #slots do
		local itemLevel, iqRed, iqGreen, iqBlue;
		local slotID = GetInventorySlotInfo(slots[i] .. "Slot");
		local itemLink = GetInventoryItemLink("player", slotID);
		if (itemLink ~= nil) then
			local itemType = select(9, GetItemInfo(itemLink));
			local item = Item:CreateFromItemLink(itemLink);
			item:ContinueOnItemLoad(function()
				itemLevel = item:GetCurrentItemLevel();
				iqRed, iqGreen, iqBlue = item:GetItemQualityColorRGB();
			end)
			local gearItem = {slots[i], itemLink, itemLevel, iqRed, iqGreen, iqBlue, itemType};
			playerGear[slotID] = gearItem;
		end
	end
end

--start: add inspect item level data to tooltips
GameTooltip:HookScript("OnUpdate", function(self)
	if ki_showTooltip then
		KhesyciLvlAddLineToTooltip(self);
	end
end);

function KhesyciLvlAddLineToTooltip(self)
	if UnitIsPlayer("mouseover") and iLvl ~= "0" then
		if KhesyciLvlVerifyScanned(KhesyciLvlGetMouseoverName()) then
			local addLine = true;

			for i = 1, self:NumLines(), 1 do
				local line = _G[self:GetName().."TextLeft"..i]:GetText()
				if line == nil then line = "" end
				if strlen(line) > 4 then
					if (strsub(line, 1, 4) == "iLvl") then addLine = false end
				end
			end

			if (addLine == true) then
				if (ki_addLine) then
					self:AddLine(" ");
				end

				if (ki_showWeapons and weapiLvl ~= "0") then
					self:AddLine("iLvl: " .. iLvl .. weapiLvl);
				else
					self:AddLine("iLvl: " .. iLvl);
				end
				self:Show();
			end
		end
	end
end
--end: add inspect item level data to tooltips

--start: hook to the character pane opening
local ki_title;
local ki_ilvl = {};
local ki_top = {};
local ki_bottom = {};
hooksecurefunc("PaperDollItemSlotButton_Update", function(button)
	--verify that we have had a successful player scan
	if #playerGear == 0 then KhesyciLvlScanPlayer() end

	--this section of code adds a label to the center point of the character above the weapons
	--local charLabel = CreateFrame("Frame");
	--charLabel:SetScript("OnUpdate", function(self)
	--	if not self.note then
	--		self.note = CharacterModelScene:CreateFontString(nil, "OVERLAY");
	--		self.note:SetPoint("BOTTOM", 0, 20);
	--		self.note:SetFontObject(NumberFontNormal);
	--		self.note:SetFormattedText("test");
	--	end
	--end)

	--this section of code adds a labels to each equipment slot
	if (button:GetID() <= 19) then
		local hasTop, hasBottom;
		if playerGear[button:GetID()] and ki_showCharacter then
			if (not ki_ilvl[button:GetID()]) then ki_ilvl[button:GetID()] = button:CreateFontString(nil, "OVERLAY") end
			ki_ilvl[button:GetID()]:SetFontObject(NumberFontNormal);
			ki_ilvl[button:GetID()]:SetPoint("TOP", 0, -3);
			if (not playerGear[button:GetID()][3]) then KhesyciLvlScanPlayer() end
			ki_ilvl[button:GetID()]:SetFormattedText(playerGear[button:GetID()][3]);
			ki_ilvl[button:GetID()]:SetTextColor(playerGear[button:GetID()][4], playerGear[button:GetID()][5], playerGear[button:GetID()][6]);
			if ki_showLevel then
				ki_ilvl[button:GetID()]:Show();
			else
				ki_ilvl[button:GetID()]:Hide();
			end

			if (not ki_top[button:GetID()]) then ki_top[button:GetID()] = {button:CreateFontString(nil, "OVERLAY"), button:CreateFontString(nil, "OVERLAY"), button:CreateFontString(nil, "OVERLAY")} end
			for i = 1, #ki_top[button:GetID()] do
				ki_top[button:GetID()][i]:SetFontObject(NumberFontNormalSmall);
			end
			local ench = enchants[button:GetID()] or 0;
			if ench and (playerGear[button:GetID()][7] == "INVTYPE_SHIELD" or playerGear[button:GetID()][7] == "INVTYPE_HOLDABLE") then ench = ench - 1 end
			local gem = gems[button:GetID()] or 0;
			local emb = ench + gem;

			local stats = GetItemStats(GetInventoryItemLink("player", button:GetID()));
			local emptySlots = 0;
			local slots = 0;
			local enchants = 0;
			for label, stat in pairs(stats) do
				if label:match("EMPTY_SOCKET_") then
					emptySlots = emptySlots + stat;
				end
			end
			local gem1, gem2, gem3, gem4 = select(4, strsplit(":", GetInventoryItemLink("player", button:GetID())));
			slots = (gem1 ~= "" and 1 or 0) + (gem2 ~= "" and 1 or 0) + (gem3 ~= "" and 1 or 0);
			local enchant = select(3, strsplit(":", GetInventoryItemLink("player", button:GetID())));
			enchants = (enchant ~= "" and 1 or 0);

			if (emb >= 1) then
				if ench == 1 then
					ki_top[button:GetID()][1]:SetFormattedText("E");
					if (enchants == 1) then
						ki_top[button:GetID()][1]:SetTextColor(0.12, 1.00, 0.00);
					else
						ki_top[button:GetID()][1]:SetTextColor(1.00, 0.00, 0.00);
					end
				else
					ki_top[button:GetID()][1]:SetFormattedText("G");
					if (slots >= 1) then
						ki_top[button:GetID()][1]:SetTextColor(0.12, 1.00, 0.00);
					elseif (emptySlots >= 1) then
						ki_top[button:GetID()][1]:SetTextColor(1.00, 0.00, 0.00);
					else
						ki_top[button:GetID()][1]:SetTextColor(0.62, 0.62, 0.62);
					end
				end
				if ki_showGems then
					ki_top[button:GetID()][1]:Show();
				else
					ki_top[button:GetID()][1]:Hide();
				end
				hasTop = ki_showGems;
			else
				ki_top[button:GetID()][1]:Hide();
				hasTop = false;
			end
			if (emb >= 2) then
				ki_top[button:GetID()][2]:SetFormattedText("G");
				if (slots >= 2 or (slots == 1 and ench == 1)) then
					ki_top[button:GetID()][2]:SetTextColor(0.12, 1.00, 0.00);
				elseif ((emptySlots >= 1 and ench == 1) or emptySlots >= 2) then
					ki_top[button:GetID()][2]:SetTextColor(1.00, 0.00, 0.00);
				else
					ki_top[button:GetID()][2]:SetTextColor(0.62, 0.62, 0.62);
				end
				if ki_showGems then
					ki_top[button:GetID()][2]:Show();
				else
					ki_top[button:GetID()][2]:Hide();
				end
			else
				ki_top[button:GetID()][2]:Hide();
			end
			if (emb == 3) then
				ki_top[button:GetID()][3]:SetFormattedText("G");
				if (slots == 3) then
					ki_top[button:GetID()][3]:SetTextColor(0.12, 1.00, 0.00);
				elseif (emptySlots >= 3) then
					ki_top[button:GetID()][3]:SetTextColor(1.00, 0.00, 0.00);
				else
					ki_top[button:GetID()][3]:SetTextColor(0.62, 0.62, 0.62);
				end
				if ki_showGems then
					ki_top[button:GetID()][3]:Show();
				else
					ki_top[button:GetID()][3]:Hide();
				end
			else
				ki_top[button:GetID()][3]:Hide();
			end

			local curr, max = GetInventoryItemDurability(button:GetID());
			if (max ~= nil) then
				if (not ki_bottom[button:GetID()]) then ki_bottom[button:GetID()] = button:CreateFontString(nil, "OVERLAY") end
				ki_bottom[button:GetID()]:SetFontObject(NumberFontNormal);
				ki_bottom[button:GetID()]:SetFormattedText(string.format("%.0f", curr / max * 100) .. "%%");
				ki_bottom[button:GetID()]:SetTextColor(playerGear[button:GetID()][4], playerGear[button:GetID()][5], playerGear[button:GetID()][6]);
				if ki_showDura then
					ki_bottom[button:GetID()]:Show();
				else
					ki_bottom[button:GetID()]:Hide();
				end
				hasBottom = ki_showDura;
			else
				if ki_bottom[button:GetID()] then ki_bottom[button:GetID()]:Hide() end
				hasBottom = false;
			end

			if hasTop and hasBottom then
				--align both rows
				if button.IsLeftSide or (button:GetID() == 17) then
					if ki_bottom[button:GetID()] then ki_bottom[button:GetID()]:SetPoint("BOTTOMRIGHT", 40, 3) end
					if (ki_top[button:GetID()] ~= nil) then
						for i = 1, #ki_top[button:GetID()] do
							ki_top[button:GetID()][i]:SetPoint("TOPRIGHT", 20 + ((i - 1) * 12), -3);
						end
					end
				else
					if ki_bottom[button:GetID()] then ki_bottom[button:GetID()]:SetPoint("BOTTOMLEFT", -40, 3) end
					if (ki_top[button:GetID()] ~= nil) then
						for i = 1, #ki_top[button:GetID()] do
							ki_top[button:GetID()][i]:SetPoint("TOPLEFT", -(20 + ((i - 1) * 12)), -3);
						end
					end
				end
			else
				--align a single row
				if button.IsLeftSide or (button:GetID() == 17) then
					if ki_bottom[button:GetID()] then ki_bottom[button:GetID()]:SetPoint("RIGHT", 40, 0) end
					if (ki_top[button:GetID()] ~= nil) then
						for i = 1, #ki_top[button:GetID()] do
							ki_top[button:GetID()][i]:SetPoint("RIGHT", 20 + ((i - 1) * 12), 0);
						end
					end
				else
					if (button:GetID() == 1) then print("Is right side") end
					if ki_bottom[button:GetID()] then ki_bottom[button:GetID()]:SetPoint("LEFT", -40, 0) end
					if (ki_top[button:GetID()] ~= nil) then
						for i = 1, #ki_top[button:GetID()] do
							ki_top[button:GetID()][i]:SetPoint("LEFT", -(20 + ((i - 1) * 12)), 0);
						end
					end
				end
			end
		else
			if ki_ilvl[button:GetID()] then ki_ilvl[button:GetID()]:Hide() end
			if ki_bottom[button:GetID()] then ki_bottom[button:GetID()]:Hide() end
			if (ki_top[button:GetID()] ~= nil) then
				for i = 1, #ki_top[button:GetID()] do
					if ki_top[button:GetID()][i] then ki_top[button:GetID()][i]:Hide() end
				end
			end
		end
	end

	--still need to find the inspect paper doll frame...
end)
--end: hook to the character pane opening